package com.ypsx.yppos.ui.activity

import android.os.Bundle
import android.text.method.HideReturnsTransformationMethod
import android.text.method.PasswordTransformationMethod
import androidx.activity.viewModels
import com.afollestad.materialdialogs.MaterialDialog
import com.afollestad.materialdialogs.list.SingleChoiceListener
import com.afollestad.materialdialogs.list.listItemsSingleChoice
import com.ypsx.base.ext.view.afterTextChange
import com.ypsx.base.ext.view.textStringTrim
import com.ypsx.base.network.manager.NetState
import com.ypsx.base.util.RegexUtils
import com.ypsx.yppos.PosApp
import com.ypsx.yppos.R
import com.ypsx.yppos.app.base.BaseActivity
import com.ypsx.yppos.app.ext.startKtxActivityFinish
import com.ypsx.yppos.databinding.ActivityLoginBinding
import com.ypsx.yppos.room.database.PosDatabase
import com.ypsx.yppos.ui.popup.AppUpdaterPopupWindow
import com.ypsx.yppos.ui.popup.MessagePopupWindow
import com.ypsx.yppos.utils.CacheUtil
import com.ypsx.yppos.utils.dXLog
import com.ypsx.yppos.utils.toast
import com.ypsx.yppos.viewmodel.request.RequestLoginViewModel
import com.ypsx.yppos.viewmodel.state.LoginViewModel

/**
 * 登录页
 */
class LoginActivity : BaseActivity<LoginViewModel, ActivityLoginBinding>() {
    private val requestLoginViewModel: RequestLoginViewModel by viewModels()
    private var exitTime: Long = 0
    private var clickNum = 1
    override fun layoutId(): Int = R.layout.activity_login
    private val hostMap = mapOf(
        "sit开发环境Host" to "https://sit-cpos.ypshengxian.com/",
        "预发Host" to "https://pre-cpos.ypshengxian.com/",
        "线上正式Host" to "https://cpos.ypshengxian.com/",
    )
    private val hosts = hostMap.values.toTypedArray()
    private val names = hostMap.keys.toList()

    override fun initView(savedInstanceState: Bundle?) {
        mDatabind.vm = mViewModel
        mDatabind.click = ProxyClick()

        mDatabind.etAccount.afterTextChange {
            val a = it.isNotEmpty() && RegexUtils.isMobileSimple(it)
            val password = mDatabind.etPassword.textStringTrim()
            val b = password.isNotEmpty() && password.length >= 6
            if (a && b) {
                mViewModel.enabledStatus.set(true)
            } else {
                mViewModel.enabledStatus.set(false)
            }
        }
        mDatabind.etPassword.afterTextChange {
            val b = it.isNotEmpty() && it.length >= 6
            val username = mDatabind.etAccount.textStringTrim()
            val a = username.isNotEmpty() && RegexUtils.isMobileSimple(username)
            if (a && b) {
                mViewModel.enabledStatus.set(true)
            } else {
                mViewModel.enabledStatus.set(false)
            }
        }
        CacheUtil.getAliYunKey().dXLog()

        requestLoginViewModel.upgradeApk(mViewModel.posId.get())

    }


    inner class ProxyClick {
        fun login() {
            val username = mDatabind.etAccount.textStringTrim()
            val password = mDatabind.etPassword.textStringTrim()
            showLoading(getString(R.string.loading))
            requestLoginViewModel.login(username, password)
        }

        fun eye() {
            val isHideFirst = mViewModel.isHideFirst.get()
            if (isHideFirst) {
                mViewModel.isHideFirst.set(false)
                mDatabind.btEye.setImageResource(R.drawable.ic_eye_open)
                //显示
                mDatabind.etPassword.transformationMethod =
                    HideReturnsTransformationMethod.getInstance()
                mDatabind.etPassword.setSelection(mDatabind.etPassword.textStringTrim().length)

            } else {
                mViewModel.isHideFirst.set(true)
                mDatabind.etPassword.transformationMethod =
                    PasswordTransformationMethod.getInstance()
                mDatabind.btEye.setImageResource(R.drawable.ic_eye_close)
                mDatabind.etPassword.setSelection(mDatabind.etPassword.textStringTrim().length)
            }
        }


        fun combosClick(){
            if (System.currentTimeMillis() - exitTime > 2000) {
                clickNum = 1
                exitTime = System.currentTimeMillis()
            } else {
                exitTime = System.currentTimeMillis()
                clickNum += 1
            }
            if (clickNum >= 5) {
                showServerHostPopWindow()
            }
        }
    }

    private fun showServerHostPopWindow() {
        //选中的host
        val checkHost = CacheUtil.getHost()
        var checkedIndex = hosts.indexOf(checkHost)
        if (checkedIndex < 0) checkedIndex = names.size - 1//也就是研发Native那个
        //启用输入的host
        MaterialDialog(this).title(text = "切换Server接口")
            .cancelable(false)
            .cancelOnTouchOutside(false)
            .listItemsSingleChoice(
                items = names,
                initialSelection = checkedIndex,
                selection = object : SingleChoiceListener {
                    override fun invoke(dialog: MaterialDialog, index: Int, text: CharSequence) {
                        //如果选择了研发native，index就会超过host的index
                        if (index < hosts.size) {
                            CacheUtil.setHost(hosts[index])
                            CacheUtil.clearAll()
                            PosDatabase.clearAllTables()
                            PosApp.instance.restartApplication()
                        }
                    }
                })
            .positiveButton(text = "确定")
            .negativeButton(text = "取消")
            .show()
    }

    override fun createObserver() {
        requestLoginViewModel.posDetails.observe(this, {
            dismissLoading()
            if (it.isSuccess) {
                startKtxActivityFinish<MainActivity>()
            }
        })
        requestLoginViewModel.posDetails2.observe(this, {
            dismissLoading()
            if (!it.isSuccess) {
                val messagePopupWindow = MessagePopupWindow(this)
                messagePopupWindow.showPopupWindow()
                messagePopupWindow.setData("提示", it.errorMsg, "取消", "确定")
                messagePopupWindow.setMessageClick {
                    val username = mDatabind.etAccount.textStringTrim()
                    showLoading(getString(R.string.loading))
                    requestLoginViewModel.posKickOff(username)
                }
            } else {
                it.errorMsg.toast()
            }
        })
        requestLoginViewModel.appVersion.observe(this,{
            val appUpdaterPopupWindow = AppUpdaterPopupWindow(this)
            appUpdaterPopupWindow.showPopupWindow()
            appUpdaterPopupWindow.setData(it)
        })

    }

    override fun onNetworkStateChanged(netState: NetState) {
        if (netState.isSuccess) {
            "我特么终于有网了啊!".dXLog()
        } else {
            "我特么怎么断网了!".dXLog()
        }
    }
}